﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using Windows.Foundation;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Animation;
using Windows.UI.Xaml.Shapes;

namespace ManipulationTracker
{
    public sealed partial class TouchDisplay
    {
        Dictionary<uint, PointerInfo> pointerDictionary = new Dictionary<uint, PointerInfo>();

        public TouchDisplay()
        {
            InitializeComponent();
        }

        protected override void OnPointerPressed(PointerEventArgs args)
        {
            Debug.WriteLine("OnPointerPressed: {0} {1}", args.Pointer.PointerDeviceType, args.Pointer.PointerId);

            Path ellipse = CreateEllipse(args.GetCurrentPoint(this).Position, Colors.Blue);
            LayoutRoot.Children.Add(ellipse);
            pointerDictionary.Add(args.Pointer.PointerId, new PointerInfo { StartFigure = ellipse });
            CapturePointer(args.Pointer);
            base.OnPointerPressed(args);
        }

        protected override void OnPointerMoved(PointerEventArgs args)
        {
            if (pointerDictionary.ContainsKey(args.Pointer.PointerId))
            {
                PointerInfo pointerInfo = pointerDictionary[args.Pointer.PointerId];

                if (pointerInfo.CurrentFigure == null)
                {
                    Path ellipse = CreateEllipse(args.GetCurrentPoint(this).Position, Colors.Red);
                    LayoutRoot.Children.Add(ellipse);
                    pointerInfo.CurrentFigure = ellipse;
                }
                else
                {
                    (pointerInfo.CurrentFigure.Data as EllipseGeometry).Center = args.GetCurrentPoint(this).Position;
                }
            }
            base.OnPointerMoved(args);
        }

        protected override void OnPointerReleased(PointerEventArgs args)
        {
            Debug.WriteLine("OnPointerReleased: {0} {1}", args.Pointer.PointerDeviceType, args.Pointer.PointerId);

            if (pointerDictionary.ContainsKey(args.Pointer.PointerId))
            {
                PointerInfo pointerInfo = pointerDictionary[args.Pointer.PointerId];
                LayoutRoot.Children.Remove(pointerInfo.StartFigure);

                if (pointerInfo.CurrentFigure != null)
                    LayoutRoot.Children.Remove(pointerInfo.CurrentFigure);

                pointerDictionary.Remove(args.Pointer.PointerId);
            }
            ReleasePointerCapture(args.Pointer);
            base.OnPointerReleased(args);
        }

        protected override void OnTapped(TappedEventArgs args)
        {
            Debug.WriteLine("OnTapped: {0}", args.PointerDeviceType);

            Point pt = args.GetPosition(this);
            tappedLine.X1 = pt.X;
            tappedLine.Y1 = pt.Y;
            pt = tappedText.TransformToVisual(this).TransformPoint(new Point(0, tappedText.ActualHeight / 2));
            tappedLine.X2 = pt.X;
            tappedLine.Y2 = pt.Y;
            (Resources["tappedStoryboard"] as Storyboard).Begin();
            base.OnTapped(args);
        }

        protected override void OnDoubleTapped(DoubleTappedEventArgs args)
        {
            Debug.WriteLine("OnDoubleTapped: {0}", args.PointerDeviceType);

            Point pt = args.GetPosition(this);
            doubleTappedLine.X1 = pt.X;
            doubleTappedLine.Y1 = pt.Y;
            pt = doubleTappedText.TransformToVisual(this).TransformPoint(new Point(0, doubleTappedText.ActualHeight / 2));
            doubleTappedLine.X2 = pt.X;
            doubleTappedLine.Y2 = pt.Y;
            (Resources["doubleTappedStoryboard"] as Storyboard).Begin();
            base.OnDoubleTapped(args);
        }

        protected override void OnRightTapped(RightTappedEventArgs args)
        {
            Debug.WriteLine("OnRightTapped: {0}", args.PointerDeviceType);

            Point pt = args.GetPosition(this);
            rightTappedLine.X1 = pt.X;
            rightTappedLine.Y1 = pt.Y;
            pt = rightTappedText.TransformToVisual(this).TransformPoint(new Point(0, rightTappedText.ActualHeight / 2));
            rightTappedLine.X2 = pt.X;
            rightTappedLine.Y2 = pt.Y;
            (Resources["rightTappedStoryboard"] as Storyboard).Begin();
            base.OnRightTapped(args);
        }

        protected override void OnHolding(HoldingEventArgs args)
        {
            Debug.WriteLine("OnHolding: {0} {1}", args.PointerDeviceType, args.HoldingState);

            Point pt = args.GetPosition(this);
            holdingLine.X1 = pt.X;
            holdingLine.Y1 = pt.Y;
            pt = holdingText.TransformToVisual(this).TransformPoint(new Point(0, holdingText.ActualHeight / 2));
            holdingLine.X2 = pt.X;
            holdingLine.Y2 = pt.Y;

            switch (args.HoldingState)
            {
                case Windows.UI.Input.HoldingState.Started:
                    holdingText.Foreground =
                    holdingLine.Stroke = new SolidColorBrush(Colors.White);
                    (Resources["startHoldingStoryboard"] as Storyboard).Begin();
                    break;

                case Windows.UI.Input.HoldingState.Completed:
                    holdingText.Foreground =
                    holdingLine.Stroke = new SolidColorBrush(Colors.White);
                    (Resources["completeHoldingStoryboard"] as Storyboard).Begin();
                    break;

                case Windows.UI.Input.HoldingState.Canceled:
                    holdingText.Foreground =
                    holdingLine.Stroke = new SolidColorBrush(Colors.Red);
                    (Resources["completeHoldingStoryboard"] as Storyboard).Begin();
                    break;
            }

            base.OnHolding(args);
        }

        protected override void OnManipulationStarting(ManipulationStartingEventArgs args)
        {
            Debug.WriteLine("OnManipulationStarting");

            // This should be fine but it raises an exception
            // args.ManipulationContainer = this;

            args.Handled = true;
            base.OnManipulationStarting(args);
        }

        protected override void OnManipulationStarted(ManipulationStartedEventArgs args)
        {
            Debug.WriteLine("OnManipulationStarted");

            // Because ManipulationContainer can't be set, 
            //      ManipulationOrigin is relative to the app, 
            //          so it must be shifted to be relative to this control.
            Point manipulationOrigin = Window.Current.Content.TransformToVisual(this).TransformPoint(args.ManipulationOrigin);

            manipulationStartedFigure.Visibility = Visibility.Visible;
            manipulationStartedTransform.X = manipulationOrigin.X;
            manipulationStartedTransform.Y = manipulationOrigin.Y;
            args.Handled = true;
            base.OnManipulationStarted(args);
        }

        protected override void OnManipulationDelta(ManipulationDeltaEventArgs args)
        {
            // Because ManipulationContainer can't be set, 
            //      ManipulationOrigin is relative to the app, 
            //          so it must be shifted to be relative to this control.
            Point manipulationOrigin = Window.Current.Content.TransformToVisual(this).TransformPoint(args.ManipulationOrigin);

            // Set the location, size, and rotation of delta rectangle
            manipulationDeltaFigure.Visibility = Visibility.Visible;
            manipulationDeltaTransform.TranslateX = manipulationOrigin.X;
            manipulationDeltaTransform.TranslateY = manipulationOrigin.Y;

            if (args.CumulativeManipulation.Scale == 0)
            {
                manipulationDeltaTransform.ScaleX = 1;
                manipulationDeltaTransform.ScaleY = 1;
            }
            else
            {
                manipulationDeltaTransform.ScaleX = 
                manipulationDeltaTransform.ScaleY = args.CumulativeManipulation.Scale;
            }

            manipulationDeltaTransform.Rotation = 180 * args.CumulativeManipulation.Rotation / Math.PI; 

            // Set the points for the translation line
            deltaTranslateFigure.Visibility = Visibility.Visible;
            deltaTranslateLine.StartPoint = new Point(manipulationOrigin.X - args.CumulativeManipulation.Translation.X,
                                                      manipulationOrigin.Y - args.CumulativeManipulation.Translation.Y);
            deltaTranslateLine.EndPoint = manipulationOrigin;

            // Set the text renditions of the manipulation
            deltaTrans.Text = String.Format("({0:F2}, {1:F2})", args.DeltaManipulation.Translation.X, 
                                                                args.DeltaManipulation.Translation.Y);

            cumulTrans.Text = String.Format("({0:F2}, {1:F2})", args.CumulativeManipulation.Translation.X, 
                                                                args.CumulativeManipulation.Translation.Y);

            deltaScale.Text = args.DeltaManipulation.Scale.ToString("F2");
            cumulScale.Text = args.CumulativeManipulation.Scale.ToString("F2");

            deltaRotate.Text = args.DeltaManipulation.Rotation.ToString("F2");
            cumulRotate.Text = args.CumulativeManipulation.Rotation.ToString("F2");

            args.Handled = true;
            base.OnManipulationDelta(args);
        }

        protected override void OnManipulationInertiaStarting(ManipulationInertiaStartingEventArgs args)
        {
            args.Handled = true;
            base.OnManipulationInertiaStarting(args);
        }

        protected override void OnManipulationCompleted(ManipulationCompletedEventArgs args)
        {
            Debug.WriteLine("OnManipulationCompleted");

            manipulationStartedFigure.Visibility = Visibility.Collapsed;
            manipulationDeltaFigure.Visibility = Visibility.Collapsed;
            deltaTranslateFigure.Visibility = Visibility.Collapsed;
            args.Handled = true;
            base.OnManipulationCompleted(args);
        }

        Path CreateEllipse(Point ptCenter, Color baseColor)
        {
            return new Path
            {
                IsHitTestVisible = false,
                Fill = new SolidColorBrush(ColorHelper.FromArgb(0x40, baseColor.R, baseColor.G, baseColor.B)),
                Data = new EllipseGeometry
                {
                    RadiusX = 24,
                    RadiusY = 24,
                    Center = ptCenter
                },
            };
        }

        class PointerInfo
        {
            public Path StartFigure { set; get; }
            public Path CurrentFigure { set; get; }
        };
    }
}
